@@ -12,6 +12,7 @@ const MainWindow = imports.mainWindow;
 const PasteManager = imports.pasteManager;
 const RoomManager = imports.roomManager;
 const TelepathyClient = imports.telepathyClient;
+const UserTracker = imports.userTracker;
 const Utils = imports.utils;
 const NetworksManager = imports.networksManager;
 
@@ -30,11 +31,12 @@ const Application = new Lang.Class({
                       flags: Gio.ApplicationFlags.HANDLES_OPEN });
 
         GLib.set_application_name('Polari');
+        GLib.set_prgname('org.gnome.Polari');
         this._retryData = new Map();
 
         this.add_main_option('start-client', 0,
                              GLib.OptionFlags.NONE, GLib.OptionArg.NONE,
-                             'Start Telephathy client', null);
+                             _("Start Telephathy client"), null);
         this.connect('handle-local-options', (o, dict) => {
             try {
                 this.register(null);
@@ -153,6 +155,7 @@ const Application = new Lang.Class({
 
         this._roomManager = RoomManager.getDefault();
         this._accountsMonitor = AccountsMonitor.getDefault();
+        this._userStatusMonitor = UserTracker.getUserStatusMonitor();
         this._networksManager = NetworksManager.getDefault();
 
         this._accountsMonitor.connect('account-status-changed',
